@@ -33,7 +33,7 @@
   else {
     phi.scale <- mcmc.input$phi.scale
     if(nmphi > 1) {
-      p.discr.rejects <- 1 - 2 * (1 - pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale)))
+      p.discr.rejects <- 1 - 2 * pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale), lower.tail = FALSE )
       if(p.discr.rejects > 0.2)
          warning(paste("probability of rejecting phi-proposal because of discr. is \n", p.discr.rejects))
     }
@@ -81,7 +81,8 @@
   }
   else{
     result <- .C("mcmcrun4",
-                 COPY = c(F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, T, T),
+                 COPY = c(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
+                   FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE),
                  as.integer(n),
                  as.double(data),
                  as.double(units.m),
@@ -146,7 +147,7 @@
   else {
     phi.scale <- mcmc.input$phi.scale
     if(nmphi > 1) {
-      p.discr.rejects <- 1 - 2 * (1 - pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale)))
+      p.discr.rejects <- 1 - 2 * pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale),  lower.tail = FALSE)
       if(p.discr.rejects > 0.2)
          warning(paste("probability of rejecting phi-proposal because of discr. is \n", p.discr.rejects))
     }
@@ -195,7 +196,8 @@
   }
   else{
     result <- .C("mcmcrun4boxcox",
-                 COPY = c(F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, T, T),
+                 COPY = c(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
+                   FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE),
                  as.integer(n),
                  as.double(data),
                  as.double(units.m),                 
@@ -262,7 +264,7 @@
   else {
     phi.scale <- mcmc.input$phi.scale
     if(nmphi > 1) {
-      p.discr.rejects <- 1 - 2 * (1 - pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale)))
+      p.discr.rejects <- 1 - 2 * pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale),  lower.tail = FALSE)
       if(p.discr.rejects > 0.2)
         warning(paste("probability of rejecting phi-proposal because of discr. is \n", p.discr.rejects))
     }
@@ -309,7 +311,8 @@
   }
   else{
     result <- .C("mcmcrun5",
-                 COPY = c(F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, T, T),
+                 COPY = c(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
+                   FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE),
                  as.integer(n),
                  as.double(data),
                  as.double(units.m),
@@ -374,7 +377,7 @@
   else {
     phi.scale <- mcmc.input$phi.scale
     if(nmphi > 1) {
-      p.discr.rejects <- 1 - 2 * (1 - pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale)))
+      p.discr.rejects <- 1 - 2 * pnorm((phi.discrete[nmphi] - phi.discrete[1])/(2 * (nmphi - 1)), sd = sqrt(phi.scale), lower.tail = FALSE )
       if(p.discr.rejects > 0.2)
         warning(paste("probability of rejecting phi-proposal because of discr. is \n", p.discr.rejects))
     }
@@ -422,7 +425,8 @@
   }
   else{
     result <- .C("mcmcrun5boxcox",
-                 COPY = c(F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, T, T),
+                 COPY = c(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
+                   FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE),
                  as.integer(n),
                  as.double(data),
                  as.double(units.m),
@@ -597,7 +601,7 @@
     }
   }
   quantile.estimator <- output$quantile.estimator
-  probability.estimator <- output$probability.estimator
+  probability.estimator <- output$probability.estimator  
   inference <- output$inference
   messages.screen <- output$messages.screen
   ## check == here
@@ -692,6 +696,7 @@
     if(beta.prior == "flat") df.model <- n - beta.size + df.sigmasq
     else df.model <- n + df.sigmasq
   }
+  else df.model <- Inf
   if(beta.prior == "normal"){
     if(beta.size > 1) ttvbetatt <- trend.data%*%beta.var%*%t(trend.data)
     else ttvbetatt <- trend.data%*%t(trend.data)*beta.var
@@ -760,7 +765,7 @@
     }
     kb.results$posterior$phi$sample <- gauss.post$phi.sample
     gauss.post <- gauss.post$Sdata
-  }
+  }  
   ## ######### removing junk #########################################
   if(is.R()){
     if(sigmasq.prior == "fixed"){
@@ -775,11 +780,11 @@
       if(beta.prior == "flat") remove("QQtinvtt", frame = sys.nframe())
     }
     remove("ttvbetatt", frame = sys.nframe())
-  }
+  }  
   ##           
 ##############-------------PART 3----------######################
 ##############------------prediction-------######################
-  ##
+  ## 
   n.sim <- ncol(gauss.post)
   if(inference) {
     temp.post <- list()
@@ -820,11 +825,11 @@
         temp.pred$var <- temp.result$predictive$variance
         if(output$sim.predict){
           if(lambda == 0) kb.results$predictive$simulations <- exp(temp.result$predictive$simulations)
-          else kb.results$predictive$simulations <- BCinv.aux(temp.result$predictive$simulations, lambda=lambda)
+          else kb.results$predictive$simulations <- BC.inv(temp.result$predictive$simulations, lambda=lambda)
         }
-      }
+      }      
     }
-    else {
+    else {    
       len.phi.discrete <- length(phi.discrete)      
       step.phi.discrete <- phi.discrete[2] - phi.discrete[1]
       phi.table <- rep(0,len.phi.discrete)
@@ -848,250 +853,176 @@
           temp.post$beta.mean[, indic.phi[i,1]] <- temp.result$posterior$beta$pars$mean
           temp.post$beta.var[,  , indic.phi[i,1]] <- temp.result$posterior$beta$pars$var
           temp.post$S2[indic.phi[i,1]] <- temp.result$posterior$sigmasq$pars$S2
-          if(do.prediction) {            
-            temp.pred$mean[, indic.phi[i, 1]] <- temp.result$predictive$mean
-            temp.pred$var[, indic.phi[i, 1]] <- temp.result$predictive$variance
+          if(do.prediction){        
+            temp.pred$mean[, indic.phi[i, 1]] <- temp.result$predictive$mean            
+            temp.pred$var[, indic.phi[i, 1]] <- temp.result$predictive$variance            
             if(output$sim.predict) {
-              if(lambda == 0) kb.results$predictive$simulations[, indic.phi[i, 1]] <- exp(temp.result$predictive$simulations)
-              else kb.results$predictive$simulations[, indic.phi[i, 1]] <- BCinv.aux(temp.result$predictive$simulations, lambda=lambda)
+              kb.results$predictive$simulations[, indic.phi[i, 1]] <- BC.inv(temp.result$predictive$simulations, lambda)
             }
           }
         }
-        else { 
+        else {
           prior.temp$phi <- phi.sample.unique[i]
           temp.result <- krige.bayes.extnd(data = gauss.post[, indic.phi[i,1:phi.table[i]]], coords = coords, locations = locations, 
                                            model = model.temp, prior = prior.temp, output = output.temp)  
           temp.post$beta.mean[, indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$mean         
           temp.post$beta.var[,  , indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$var
-          temp.post$S2[indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$sigmasq$pars$S2       
+          temp.post$S2[indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$sigmasq$pars$S2
           if(do.prediction) {
             temp.pred$mean[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$mean
-            temp.pred$var[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$variance
+            temp.pred$var[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$variance          
             if(output$sim.predict) {
-              if(lambda == 0) kb.results$predictive$simulations[ , indic.phi[i, 1:phi.table[i]]] <- exp(temp.result$predictive$simulations)
-              else kb.results$predictive$simulations[ , indic.phi[i, 1:phi.table[i]]] <- BCinv.aux(temp.result$predictive$simulations, lambda=lambda)  
+              kb.results$predictive$simulations[ , indic.phi[i, 1:phi.table[i]]] <- BC.inv(temp.result$predictive$simulations, lambda)  
             }
           }
         }
       }
-    }
-    if(is.R())
-      remove("temp.result")
+    }    
+    if(is.R()) remove("temp.result")
     else remove("temp.result", frame = sys.nframe())
     if(do.prediction) {
       ##
       d0mat <- loccoords(coords, locations)
       loc.coincide <- (apply(d0mat < 1e-10, 2, sum) == 1)
-      if(any(loc.coincide))
-        stop("locations coincident with datapoints are not allowed") # NEEDS TO BE SOLVED      
       ##
       ##------- calculating medians and uncertainty
       ##
-      temp.mean <- apply(temp.pred$mean, 1, median)
+      temp.med <- apply(temp.pred$mean, 1, median)
       temp.unc <- sqrt(apply(temp.pred$mean, 1, var) + apply(temp.pred$var, 1, median))
-      if(sigmasq.prior == "fixed") {
-        diffe <- cut0.calc.mixed.gauss(temp.mean,temp.pred)-0.5
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        parms.temp <- list()
-        while(any(not.accurate)){
-          temp.mean <- temp.mean - sqrt(2*3.1415)*temp.unc*diffe
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.gauss(temp.mean[not.accurate], parms.temp)-0.5
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE) 
-        }      
-        temp.upper <- temp.mean + 2 * temp.unc
-        diffe <- cut0.calc.mixed.gauss(temp.upper,temp.pred)-0.975
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        while(any(not.accurate)) {
-          if(lambda == 0) temp.upper <- temp.upper - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*1.96^2)
-          else temp.upper <- temp.upper - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*1.96^2, lambda=lambda) 
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.gauss(temp.upper[not.accurate], parms.temp)-0.975
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)  
-        }
-        temp.lower <- temp.mean - 2 * temp.unc
-        diffe <- cut0.calc.mixed.gauss(temp.lower,temp.pred)-0.025
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        while(any(not.accurate)){
-          if(lambda == 0) temp.lower <- temp.lower - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*1.96^2)
-          else temp.lower <- temp.lower - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*1.96^2, lambda=lambda)
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.gauss(temp.lower[not.accurate],parms.temp)-0.025
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        }
-        if(lambda == 0){
-          kb.results$predictive$median <- exp(temp.mean)
-          kb.results$predictive$uncertainty <- (exp(temp.upper) - exp(temp.lower))/4
-        }
-        else{
-          kb.results$predictive$median <- BCinv.aux(temp.mean, lambda=lambda)
-          kb.results$predictive$uncertainty <- (BCinv.aux(temp.upper, lambda=lambda) - BCinv.aux(temp.lower, lambda=lambda))/4
-        }
-        if(is.null(quantile.estimator)) {
-          if(lambda == 0) kb.results$predictive$quantiles <- as.data.frame(cbind(exp(temp.lower), exp(temp.mean), exp(temp.upper)))
-          else kb.results$predictive$quantiles <- as.data.frame(cbind(BCinv.aux(temp.lower, lambda=lambda), BCinv.aux(temp.mean, lambda=lambda), BCinv.aux(temp.upper, lambda=lambda)))
-          qname <- rep(0, 3)
-          qname[1] <- paste("q0.025", sep = "")
-          qname[2] <- paste("q0.5", sep = "")
-          qname[3] <- paste("q0.975", sep = "")
-          names(kb.results$predictive$quantiles) <- qname
-        }
-        if(is.numeric(quantile.estimator)) {
-          nmq <- length(quantile.estimator)
-          if(nmq > 1) {
-            temp.quan <- matrix(NA, ni, nmq)
-            dig <- rep(3, nmq)
-            for(i in 1:nmq) {
-              while(quantile.estimator[i] != round(quantile.estimator[i], digits = dig[i])) dig[i] <-dig[i] + 1
-              temp.quan[, i] <- qnorm(rep(quantile.estimator[i], ni), mean= temp.mean,sd = temp.unc)
-              diffe <- cut0.calc.mixed.gauss(temp.quan[ , i], temp.pred)-quantile.estimator[i]
-              not.accurate <- ifelse(abs(diffe)>0.5/10^dig[i], TRUE, FALSE)
-              while(any(not.accurate)) {
-                if(lambda == 0) temp.quan[, i] <- temp.quan[ , i] - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*qnorm(quantile.estimator[i]))
-                else temp.quan[, i] <- temp.quan[ , i] - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*qnorm(quantile.estimator[i]), lambda=lambda)
-                diffe <- cut0.calc.mixed.gauss(temp.quan[, i],temp.pred)-quantile.estimator[i]
-                not.accurate <- ifelse(abs(diffe)>0.5/10^dig[i], TRUE, FALSE)
-              }
-            }
-            if(lambda == 0) kb.results$predictive$quantiles <- as.data.frame(exp(temp.quan))
-            else kb.results$predictive$quantiles <- as.data.frame(BCinv.aux(temp.quan, lambda=lambda))
-            qname <- rep(0, length(quantile.estimator))
-            for(i in 1:length(quantile.estimator))
-              qname[i] <- paste("q", 100 * quantile.estimator[i], sep = "")
-            names(kb.results$predictive$quantiles) <- qname
-          }
-          else {
-            dig <- 3
-            while(quantile.estimator != round(quantile.estimator,digits = dig)) dig <-dig + 1
-            temp.quan <- qnorm(rep(quantile.estimator,ni), mean = temp.mean, sd = temp.unc)
-            diffe <- cut0.calc.mixed.gauss(temp.quan, temp.pred)-quantile.estimator
-            not.accurate <- ifelse(abs(diffe)>0.5/10^dig, TRUE, FALSE)
+      not.accurate <- (!loc.coincide)
+      diffe <- pmixed(temp.med, temp.pred,df.model)-0.5
+      temp.med.new <- temp.med[not.accurate]+0.1*(temp.med[not.accurate]+0.1) # to get started
+      inv.sl <- rep(0,ni)
+      parms.temp <- list()
+      while(any(not.accurate)){
+        parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
+        parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
+        diffe.new <- pmixed(temp.med.new, parms.temp,df.model)-0.5
+        inv.sl[not.accurate] <- (temp.med.new-temp.med[not.accurate])/(diffe.new-diffe[not.accurate])
+        temp.med[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.med.new,temp.med[not.accurate])
+        diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
+        not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
+        temp.med.new <- temp.med[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
+      }
+      temp.upper <- qnorm(rep(0.975, ni), mean = temp.med, sd = temp.unc)
+      not.accurate <- (!loc.coincide)
+      diffe <- pmixed(temp.upper, temp.pred,df.model)-0.975
+      temp.upper.new <- temp.upper[not.accurate]+0.5*(temp.upper[not.accurate]+0.5) # to get started
+      inv.sl <- rep(0,ni)      
+      while(any(not.accurate)){
+        parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
+        parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
+        diffe.new <- pmixed(temp.upper.new, parms.temp,df.model)-0.975
+        inv.sl[not.accurate] <- (temp.upper.new-temp.upper[not.accurate])/(diffe.new-diffe[not.accurate])
+        temp.upper[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.upper.new,temp.upper[not.accurate])
+        diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
+        not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
+        temp.upper.new <- temp.upper[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
+      }      
+      temp.lower <- qnorm(rep(0.025, ni), mean = temp.med, sd = temp.unc)
+      not.accurate <- (!loc.coincide)
+      diffe <- pmixed(temp.lower, temp.pred,df.model)-0.025
+      temp.lower.new <- temp.lower[not.accurate]+0.5*(temp.lower[not.accurate]+0.5) # to get started
+      inv.sl <- rep(0,ni)
+      while(any(not.accurate)){
+        parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
+        parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
+        diffe.new <- pmixed(temp.lower.new,parms.temp,df.model)-0.025
+        inv.sl[not.accurate] <- (temp.lower.new-temp.lower[not.accurate])/(diffe.new-diffe[not.accurate])
+        temp.lower[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.lower.new,temp.lower[not.accurate])
+        diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
+        not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
+        temp.lower.new <- temp.lower[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
+      }
+      if(any(loc.coincide)){
+        temp.med[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, median)
+        temp.upper[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = 0.975)
+        temp.lower[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = 0.025) 
+      }
+      kb.results$predictive$median <- BC.inv(temp.med,lambda)
+      kb.results$predictive$uncertainty <- (BC.inv(temp.upper,lambda) - BC.inv(temp.lower,lambda))/4
+      ## calculating quantiles
+      if(is.null(quantile.estimator)) {
+        kb.results$predictive$quantiles <- as.data.frame(cbind(BC.inv(temp.lower,lambda), BC.inv(temp.med,lambda), BC.inv(temp.upper,lambda)))
+      }
+      if(is.numeric(quantile.estimator)) {
+        nmq <- length(quantile.estimator)
+        if(nmq > 1) {
+          temp.quan <- matrix(NA, ni, nmq)
+          dig <- rep(3, nmq)
+          for(i in 1:nmq) {
+            while(quantile.estimator[i] != round(quantile.estimator[i], digits = dig[i])) dig[i] <-dig[i] + 1
+            temp.quan[, i] <- qnorm(rep(quantile.estimator[i], ni), mean = temp.med, sd = temp.unc)
+            if(any(loc.coincide)) temp.quan[loc.coincide, i] <- temp.med[loc.coincide]
+            not.accurate <- (!loc.coincide)
+            diffe <- pmixed(temp.quan[,i], temp.pred,df.model)-quantile.estimator[i]
+            numb <- 0.1+abs(quantile.estimator[i]-0.5)
+            temp.quan.new <- temp.quan[not.accurate,i]+numb*(temp.quan[not.accurate,i]+numb) # to get started
+            inv.sl <- rep(0,ni)
             while(any(not.accurate)) {
-              if(lambda == 0) temp.quan <- temp.quan - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*qnorm(quantile.estimator))
-              else temp.quan <- temp.quan - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*qnorm(quantile.estimator), lambda=lambda)
-              diffe <- cut0.calc.mixed.gauss(temp.quan,temp.pred)-quantile.estimator
-              not.accurate <- ifelse(abs(diffe)>0.5/10^dig, TRUE, FALSE)
+              parms.temp$mean <-temp.pred$mean[not.accurate,,drop=FALSE]
+              parms.temp$var <-temp.pred$var[not.accurate,,drop=FALSE]
+              diffe[not.accurate] <- pmixed(temp.quan.new,parms.temp,df.model)-quantile.estimator[i]
+              inv.sl[not.accurate] <- (temp.quan.new-temp.quan[not.accurate, i])/(diffe.new-diffe[not.accurate])
+              temp.quan[not.accurate, i] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.quan.new,temp.quan[not.accurate, i])
+              diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
+              not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
+              temp.quan.new <- temp.quan[not.accurate, i] - diffe[not.accurate]*inv.sl[not.accurate]
+            }            
+            if(any(loc.coincide)){
+              temp.quan[loc.coincide,i] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = quantile.estimator[i])
             }
-            if(lambda == 0) kb.results$predictive$quantiles <- as.vector(exp(temp.quan))
-            else kb.results$predictive$quantiles <- as.vector(BCinv.aux(temp.quan, lambda=lambda)) 
           }
-          if(is.R())
-            remove("temp.quan")
-          else
-            remove("temp.quan", frame = sys.nframe())
+          kb.results$predictive$quantiles <- as.data.frame(BC.inv(temp.quan,lambda))
+        }
+        else {
+          dig <- 3
+          while(quantile.estimator != round(quantile.estimator,digits = dig)) dig <- dig + 1
+          temp.quan <- qnorm(rep(quantile.estimator,ni), mean = temp.med, sd = temp.unc)
+          not.accurate <- (!loc.coincide)
+          diffe <- pmixed(temp.quan, temp.pred,df.model)-quantile.estimator
+          numb <- 0.1+abs(quantile.estimator-0.5)
+          temp.quan.new <- temp.quan[not.accurate]+numb*(temp.quan[not.accurate]+numb) # to get started
+          inv.sl <- rep(0,ni)
+          while(any(not.accurate)) {
+            parms.temp$mean <-temp.pred$mean[not.accurate,,drop=FALSE]
+            parms.temp$var <-temp.pred$var[not.accurate,,drop=FALSE]
+            diffe[not.accurate] <- pmixed(temp.quan.new,parms.temp,df.model)-quantile.estimator
+            inv.sl[not.accurate] <- (temp.quan.new-temp.quan[not.accurate])/(diffe.new-diffe[not.accurate])
+            temp.quan[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.quan.new,temp.quan[not.accurate])
+            diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
+            not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
+            temp.quan.new <- temp.quan[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
+          }
+          if(any(loc.coincide)){
+            temp.quan[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = quantile.estimator)
+          }
+          kb.results$predictive$quantiles <- as.vector(BC.inv(temp.quan,lambda))
         }
       }
-      else {
-        diffe <- cut0.calc.mixed.t(temp.mean,temp.pred, df.model)-0.5        
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        parms.temp <- list()
-        while(any(not.accurate)) {  
-          temp.mean <- temp.mean - sqrt(2*3.1415)*temp.unc*diffe
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.t(temp.mean[not.accurate], parms.temp, df.model)-0.5
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)  
-        }
-        temp.upper <- temp.mean + 2 * temp.unc
-        diffe <- cut0.calc.mixed.t(temp.upper,temp.pred, df.model)-0.975
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        while(any(not.accurate)) {
-          if(lambda == 0) temp.upper <- temp.upper - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*1.96^2)
-          else temp.upper <- temp.upper - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*1.96^2, lambda=lambda)         
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.t(temp.upper[not.accurate], parms.temp, df.model)-0.975
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)          
-        }
-        temp.lower <- temp.mean - 2 * temp.unc
-        diffe <- cut0.calc.mixed.t(temp.lower,temp.pred, df.model)-0.025
-        not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)
-        while(any(not.accurate)) { 
-          if(lambda == 0) temp.lower <- temp.lower - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*1.96^2)
-          else temp.lower <- temp.lower - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*1.96^2, lambda=lambda)
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe[not.accurate] <- cut0.calc.mixed.t(temp.lower[not.accurate], parms.temp, df.model)-0.025
-          not.accurate <- ifelse(abs(diffe)>0.0005, TRUE, FALSE)  
-        }
-        if(lambda == 0){
-          kb.results$predictive$median <- exp(temp.mean)
-          kb.results$predictive$uncertainty <- (exp(temp.upper) - exp(temp.lower))/4
-        }
-        else{
-          kb.results$predictive$median <- BCinv.aux(temp.mean, lambda=lambda)
-          kb.results$predictive$uncertainty <- (BCinv.aux(temp.upper, lambda=lambda) - BCinv.aux(temp.lower, lambda=lambda))/4
-        } 
-        if(is.null(quantile.estimator)) {
-          if(lambda == 0) kb.results$predictive$quantiles <- as.data.frame(cbind(exp(temp.lower), exp(temp.mean), exp(temp.upper)))
-          else kb.results$predictive$quantiles <- as.data.frame(cbind(BCinv.aux(temp.lower, lambda=lambda), BCinv.aux(temp.mean, lambda=lambda), BCinv.aux(temp.upper, lambda=lambda))) 
-          qname <- rep(0, 3)
-          qname[1] <- paste("q0.025", sep = "")
-          qname[2] <- paste("q0.5", sep = "")
-          qname[3] <- paste("q0.975", sep = "")
-          names(kb.results$predictive$quantiles) <- qname
-        }     
-        if(is.numeric(quantile.estimator)) {
-          nmq <- length(quantile.estimator)
-          if(nmq > 1) {
-            temp.quan <- matrix(NA, ni, nmq)
-            dig <- rep(3, nmq)
-            for(i in 1:nmq) {
-              while(quantile.estimator[i] != round(quantile.estimator[i], digits = dig[i])) dig[i] <- dig[i] + 1
-              temp.quan[, i] <- qnorm(rep(quantile.estimator[i], ni), mean= temp.mean,sd = temp.unc)
-              diffe <- cut0.calc.mixed.t(temp.quan[ , i], temp.pred, df.model)-quantile.estimator[i]
-              not.accurate <- ifelse(abs(diffe)>0.5/10^dig[i], TRUE, FALSE)
-              while(any(not.accurate)) {  
-                if(lambda == 0) temp.quan[, i] <- temp.quan[ , i] - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*qnorm(quantile.estimator[i]))
-                else temp.quan[, i] <- temp.quan[ , i] - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*qnorm(quantile.estimator[i]), lambda=lambda)               
-                diffe <- cut0.calc.mixed.t(temp.quan[, i],temp.pred,df.model)-quantile.estimator[i]
-                not.accurate <- ifelse(abs(diffe)>0.5/10^dig[i], TRUE, FALSE)
-              }
-            }         
-            if(lambda == 0) kb.results$predictive$quantiles <- as.data.frame(exp(temp.quan))
-            else kb.results$predictive$quantiles <- as.data.frame(BCinv.aux(temp.quan, lambda=lambda))
-            qname <- rep(0, length(quantile.estimator))
-            for(i in 1:length(quantile.estimator))
-              qname[i] <- paste("q", 100 * quantile.estimator[i], sep = "")
-            names(kb.results$predictive$quantiles) <- qname
-          }
-          else {
-            dig <- 3
-            while(quantile.estimator != round(quantile.estimator,digits = dig)) dig <-dig + 1
-            temp.quan <- qnorm(rep(quantile.estimator,ni), mean = temp.mean, sd = temp.unc)
-            diffe <- cut0.calc.mixed.t(temp.quan, temp.pred)-quantile.estimator
-            not.accurate <- ifelse(abs(diffe)>0.5/10^dig, TRUE, FALSE) 
-            while(any(not.accurate)) {
-              if(lambda == 0) temp.quan <- temp.quan - sqrt(2*3.1415)*temp.unc*diffe*exp(0.5*qnorm(quantile.estimator))
-              else temp.quan <- temp.quan - sqrt(2*3.1415)*temp.unc*diffe*BCinv.aux(0.5*qnorm(quantile.estimator), lambda=lambda)
-              diffe <- cut0.calc.mixed.t(temp.quan,temp.pred)-quantile.estimator
-              not.accurate <- ifelse(abs(diffe)>0.5/10^dig, TRUE, FALSE)  
-            }
-            if(lambda == 0) kb.results$predictive$quantiles <- as.vector(exp(temp.quan))
-            else kb.results$predictive$quantiles <- as.vector(BCinv.aux(temp.quan, lambda=lambda)) 
-          }
-          if(is.R()) remove("temp.quan")
-          else remove("temp.quan", frame = sys.nframe())
-        }
+      if(is.null(quantile.estimator)){
+        qname <- rep(0, 3)
+        qname[1] <- paste("q0.025", sep = "")
+        qname[2] <- paste("q0.5", sep = "")
+        qname[3] <- paste("q0.975", sep = "")
+        names(kb.results$predictive$quantiles) <- qname
+      }
+      if(is.numeric(quantile.estimator) && nmq > 1) {
+        qname <- rep(0, length(quantile.estimator))
+        for(i in 1:length(quantile.estimator))
+          qname[i] <- paste("q", 100 * quantile.estimator[i], sep = "")
+        names(kb.results$predictive$quantiles) <- qname
       }
-      if(is.R()) remove(list = c("temp.mean", "temp.unc", "temp.upper","temp.lower"))
-      else remove(list = c("temp.mean", "temp.unc", "temp.upper","temp.lower"), frame = sys.nframe())
       ##
-      ## ------ probability estimators    
+      ## ------ probability estimators
       ##
-      if(!is.null(probability.estimator)) {       
+      if(!is.null(probability.estimator)) {
         if(lambda == 0) transf.probab <- ifelse(probability.estimator > 0, log(probability.estimator), -1e+17)
         else transf.probab <- ifelse(probability.estimator > 0, (probability.estimator^lambda-1)/lambda, -1e+17)
         if(length(transf.probab) == 1) transf.probab <- as.vector(transf.probab)
         for (ii in 1:length(transf.probab)){
           thresh.vec <- rep(transf.probab[ii],ni)
-          if(sigmasq.prior == "fixed")
-            kb.results$predictive$probability <- round(cut0.calc.mixed.gauss(thresh.vec, temp.pred), digits = 3)
-          else
-            kb.results$predictive$probability <- round(cut0.calc.mixed.t(thresh.vec, temp.pred, df.model), digits = 3)
+          kb.results$predictive$probability <- round(pmixed(thresh.vec, temp.pred, df.model), digits = 3) 
         }
       }
       if(is.R()) remove("temp.pred")
@@ -1124,7 +1055,7 @@
     else{
       kb.results$posterior$phi$mean <- mean(kb.results$posterior$phi$sample)
       kb.results$posterior$phi$var <- var(kb.results$posterior$phi$sample)
-    }
+    }    
     ##
     ## Simulations from the posterior of parameters.
     ##
@@ -1160,10 +1091,7 @@
     if(is.R()) remove("temp.post")
     else remove("temp.post", frame = sys.nframe())
   }
-  if(output$keep.mcmc.sim){
-    if(lambda == 0) kb.results$posterior$simulations <- exp(gauss.post)
-    else kb.results$posterior$simulations <- BCinv.aux(gauss.post, lambda=lambda)
-  }
+  if(output$keep.mcmc.sim) kb.results$posterior$simulations <- BC.inv(gauss.post, lambda)
   kb.results$model <- model
   kb.results$prior <- prior$priors.info
   kb.results$mcmc.input <- mcmc.input
@@ -1174,4 +1102,3 @@
   if(messages.screen) cat("pois.krige.bayes: done!\n")
   return(kb.results)
 }
-
